window.Gleam.initSearch([{"doc":"decipher","title":"decipher","content":"# decipher\n\n[![Package Version](https://img.shields.io/hexpm/v/modem)](https://hex.pm/packages/decipher)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/decipher/)\n\n&gt; **decipher**: to discover the meaning of something written badly or in a difficult\n&gt; or hidden way: – [Cambridge Dictionary](https://dictionary.cambridge.org/dictionary/english/decipher)\n\nDecipher is a collection of handy [`gleam/dynamic`](https://hexdocs.pm/gleam_stdlib/gleam/dynamic.html)\ndecoders to help you deal with awkward JSON APIs, impose additional constraints\non dynamic data, or decode values of more exotic types.\n","url":"index.html"},{"doc":"decipher","title":"all","content":"pub fn all(\n  decoders: List(fn(Dynamic) -&gt; Result(a, List(DecodeError))),\n) -&gt; fn(Dynamic) -&gt; Result(List(a), List(DecodeError))\n Create a decoder for a list of values from a list of decoders to run. Each\n decoder will run against the input value, and all must succeed for the decoder\n to succeed.\n\n Errors from each decoder will be collected, which means the entire list is\n run even if one decoder fails!\n\nSynonyms:\ndecipher.all\ndecipher all","url":"decipher.html#all"},{"doc":"decipher","title":"arraylike","content":"pub fn arraylike(\n  decoder: fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n) -&gt; fn(Dynamic) -&gt; Result(List(a), List(DecodeError))\n In JavaScript certain objects are said to be &quot;arraylike&quot;. These are objects\n that satisfy the following conditions:\n\n - They have a `length` property that is a non-negative integer.\n - They have a property for each integer index from `0` up to `length - 1`.\n\n Operations like `document.querySelectorAll` or `document.getElementsByTagName`\n return arraylike objects like a [`NodeList`](https://developer.mozilla.org/en-US/docs/Web/API/NodeList).\n This decoder is capable of decoding such objects into a proper Gleam `List`.\n\nSynonyms:\ndecipher.arraylike\ndecipher arraylike","url":"decipher.html#arraylike"},{"doc":"decipher","title":"at","content":"pub fn at(\n  path: List(String),\n  decoder: fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n) -&gt; fn(Dynamic) -&gt; Result(a, List(DecodeError))\n Decode a value at a given key path. This decoder is permissive and will use\n [`index`](#index) for any numeric string keys where possible.\n\nSynonyms:\ndecipher.at\ndecipher at","url":"decipher.html#at"},{"doc":"decipher","title":"base16","content":"pub fn base16(\n  dynamic: Dynamic,\n) -&gt; Result(BitArray, List(DecodeError))\n Decode a string representing base16-encoded binary data into a `BitArray`.\n\nSynonyms:\ndecipher.base16\ndecipher base16","url":"decipher.html#base16"},{"doc":"decipher","title":"base64","content":"pub fn base64(\n  dynamic: Dynamic,\n) -&gt; Result(BitArray, List(DecodeError))\n Decode a string representing base64-encoded binary data into a `BitArray`.\n\nSynonyms:\ndecipher.base64\ndecipher base64","url":"decipher.html#base64"},{"doc":"decipher","title":"base64_url_encoded","content":"pub fn base64_url_encoded(\n  dynamic: Dynamic,\n) -&gt; Result(BitArray, List(DecodeError))\n Decode a string representing url-safe base64-encoded binary data into a\n `BitArray`.\n\nSynonyms:\ndecipher.base64_url_encoded\ndecipher base64_url_encoded","url":"decipher.html#base64_url_encoded"},{"doc":"decipher","title":"bool_string","content":"pub fn bool_string(\n  dynamic: Dynamic,\n) -&gt; Result(Bool, List(DecodeError))\n Decode a string that represents a YAML-style boolean value. Any of the following\n values will be decoded as `True`:\n\n - &quot;true&quot;\n - &quot;True&quot;\n - &quot;on&quot;\n - &quot;On&quot;\n - &quot;yes&quot;\n - &quot;Yes&quot;\n\n Any of the following values will be decoded as `False`:\n\n - &quot;false&quot;\n - &quot;False&quot;\n - &quot;off&quot;\n - &quot;Off&quot;\n - &quot;no&quot;\n - &quot;No&quot;\n\n Anything else will fail to decode.\n\nSynonyms:\ndecipher.bool_string\ndecipher bool_string","url":"decipher.html#bool_string"},{"doc":"decipher","title":"enum","content":"pub fn enum(\n  variants: List(#(String, a)),\n) -&gt; fn(Dynamic) -&gt; Result(a, List(DecodeError))\n A simplified version of the [tagged_union](#tagged_union) decoder. First\n decodes a string, and then attempts to find a corresponding value from a\n list of variants.\n\n This is how the [`bool_string`](#bool_string) decoder is implemented:\n\n \n import decipher\n import gleam/dynamic.{type DecodeError, type Decoder, type Dynamic, DecodeError}\n\n pub fn bool_string(dynamic: Dynamic) -&gt; Result(Bool, List(DecodeError)) {\n   decipher.enum([\n     #(&quot;true&quot;, True),\n     #(&quot;True&quot;, True),\n     #(&quot;on&quot;, True),\n     #(&quot;On&quot;, True),\n     #(&quot;yes&quot;, True),\n     #(&quot;Yes&quot;, True),\n     #(&quot;false&quot;, False),\n     #(&quot;False&quot;, False),\n     #(&quot;off&quot;, False),\n     #(&quot;Off&quot;, False),\n     #(&quot;no&quot;, False),\n     #(&quot;No&quot;, False),\n   ])(dynamic)\n }\n \n\nSynonyms:\ndecipher.enum\ndecipher enum","url":"decipher.html#enum"},{"doc":"decipher","title":"exact_object1","content":"pub fn exact_object1(\n  constructor: fn(a) -&gt; b,\n  field1: #(String, fn(Dynamic) -&gt; Result(a, List(DecodeError))),\n) -&gt; fn(Dynamic) -&gt; Result(b, List(DecodeError))\n Decode an object with exactly one field into some value. If additional fields\n are present, decoding will fail and the additional fields will be reported in\n the error.\n\nSynonyms:\ndecipher.exact_object1\ndecipher exact_object1","url":"decipher.html#exact_object1"},{"doc":"decipher","title":"exact_object2","content":"pub fn exact_object2(\n  constructor: fn(a, b) -&gt; c,\n  field1: #(String, fn(Dynamic) -&gt; Result(a, List(DecodeError))),\n  field2: #(String, fn(Dynamic) -&gt; Result(b, List(DecodeError))),\n) -&gt; fn(Dynamic) -&gt; Result(c, List(DecodeError))\n Decode an object with exactly two fields into some value. If additional fields\n are present, decoding will fail and the additional fields will be reported in\n the error.\n\nSynonyms:\ndecipher.exact_object2\ndecipher exact_object2","url":"decipher.html#exact_object2"},{"doc":"decipher","title":"exact_object3","content":"pub fn exact_object3(\n  constructor: fn(a, b, c) -&gt; d,\n  field1: #(String, fn(Dynamic) -&gt; Result(a, List(DecodeError))),\n  field2: #(String, fn(Dynamic) -&gt; Result(b, List(DecodeError))),\n  field3: #(String, fn(Dynamic) -&gt; Result(c, List(DecodeError))),\n) -&gt; fn(Dynamic) -&gt; Result(d, List(DecodeError))\n Decode an object with exactly three fields into some value. If additional fields\n are present, decoding will fail and the additional fields will be reported in\n the error.\n\nSynonyms:\ndecipher.exact_object3\ndecipher exact_object3","url":"decipher.html#exact_object3"},{"doc":"decipher","title":"exact_object4","content":"pub fn exact_object4(\n  constructor: fn(a, b, c, d) -&gt; e,\n  field1: #(String, fn(Dynamic) -&gt; Result(a, List(DecodeError))),\n  field2: #(String, fn(Dynamic) -&gt; Result(b, List(DecodeError))),\n  field3: #(String, fn(Dynamic) -&gt; Result(c, List(DecodeError))),\n  field4: #(String, fn(Dynamic) -&gt; Result(d, List(DecodeError))),\n) -&gt; fn(Dynamic) -&gt; Result(e, List(DecodeError))\n Decode an object with exactly four fields into some value. If additional fields\n are present, decoding will fail and the additional fields will be reported in\n the error.\n\nSynonyms:\ndecipher.exact_object4\ndecipher exact_object4","url":"decipher.html#exact_object4"},{"doc":"decipher","title":"exact_object5","content":"pub fn exact_object5(\n  constructor: fn(a, b, c, d, e) -&gt; f,\n  field1: #(String, fn(Dynamic) -&gt; Result(a, List(DecodeError))),\n  field2: #(String, fn(Dynamic) -&gt; Result(b, List(DecodeError))),\n  field3: #(String, fn(Dynamic) -&gt; Result(c, List(DecodeError))),\n  field4: #(String, fn(Dynamic) -&gt; Result(d, List(DecodeError))),\n  field5: #(String, fn(Dynamic) -&gt; Result(e, List(DecodeError))),\n) -&gt; fn(Dynamic) -&gt; Result(f, List(DecodeError))\n Decode an object with exactly five fields into some value. If additional fields\n are present, decoding will fail and the additional fields will be reported in\n the error.\n\nSynonyms:\ndecipher.exact_object5\ndecipher exact_object5","url":"decipher.html#exact_object5"},{"doc":"decipher","title":"exact_object6","content":"pub fn exact_object6(\n  constructor: fn(a, b, c, d, e, f) -&gt; g,\n  field1: #(String, fn(Dynamic) -&gt; Result(a, List(DecodeError))),\n  field2: #(String, fn(Dynamic) -&gt; Result(b, List(DecodeError))),\n  field3: #(String, fn(Dynamic) -&gt; Result(c, List(DecodeError))),\n  field4: #(String, fn(Dynamic) -&gt; Result(d, List(DecodeError))),\n  field5: #(String, fn(Dynamic) -&gt; Result(e, List(DecodeError))),\n  field6: #(String, fn(Dynamic) -&gt; Result(f, List(DecodeError))),\n) -&gt; fn(Dynamic) -&gt; Result(g, List(DecodeError))\n Decode an object with exactly six fields into some value. If additional fields\n are present, decoding will fail and the additional fields will be reported in\n the error.\n\nSynonyms:\ndecipher.exact_object6\ndecipher exact_object6","url":"decipher.html#exact_object6"},{"doc":"decipher","title":"exact_set","content":"pub fn exact_set(\n  decoder: fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n) -&gt; fn(Dynamic) -&gt; Result(Set(a), List(DecodeError))\n Decode a list or [arraylike](#arraylike) into a `Set`. This decoder is slightly\n slower than the [set](#set) decoder, but it will guarantee that there were no\n duplicate values in the incoming list.\n\nSynonyms:\ndecipher.exact_set\ndecipher exact_set","url":"decipher.html#exact_set"},{"doc":"decipher","title":"float_string","content":"pub fn float_string(\n  dynamic: Dynamic,\n) -&gt; Result(Float, List(DecodeError))\n Decode a `Float` that has been converted to a string. Some JSON APIs will\n send numbers as strings, so this decoder can come in handy more often than\n you&#39;d think!\n\nSynonyms:\ndecipher.float_string\ndecipher float_string","url":"decipher.html#float_string"},{"doc":"decipher","title":"http_date","content":"pub fn http_date(\n  dynamic: Dynamic,\n) -&gt; Result(Time, List(DecodeError))\n Decode a string representing a [HTTP-date](https://www.rfc-editor.org/rfc/rfc9110#http.date)\n as a [`Time`](https://hexdocs.pm/birl/birl.html#Time) value from the birl\n package.\n\nSynonyms:\ndecipher.http_date\ndecipher http_date","url":"decipher.html#http_date"},{"doc":"decipher","title":"index","content":"pub fn index(\n  idx: Int,\n  decoder: fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n) -&gt; fn(Dynamic) -&gt; Result(a, List(DecodeError))\n Decode a value at a given index. This decoder is permissive and will try to\n decode tuples, objects with string integer keys, and in the worst case will\n decode a list and index into that.\n\n For strict tuple access, use the [`element`](https://hexdocs.pm/gleam_stdlib/gleam/dynamic.html#element)\n decoder from the standard library.\n\nSynonyms:\ndecipher.index\ndecipher index","url":"decipher.html#index"},{"doc":"decipher","title":"int_string","content":"pub fn int_string(\n  dynamic: Dynamic,\n) -&gt; Result(Int, List(DecodeError))\n Decode an `Int` that has been converted to a string. Some JSON APIs will\n send numbers as strings, so this decoder can come in handy more often than\n you&#39;d think!\n\nSynonyms:\ndecipher.int_string\ndecipher int_string","url":"decipher.html#int_string"},{"doc":"decipher","title":"iso_8601","content":"pub fn iso_8601(\n  dynamic: Dynamic,\n) -&gt; Result(Time, List(DecodeError))\n Decode a string representing an [ISO 8601 datetime](https://en.wikipedia.org/wiki/ISO_8601)\n as a [`Time`](https://hexdocs.pm/birl/birl.html#Time) value from the birl\n package.\n\nSynonyms:\ndecipher.iso_8601\ndecipher iso_8601","url":"decipher.html#iso_8601"},{"doc":"decipher","title":"json_string","content":"pub fn json_string(\n  decoder: fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n) -&gt; fn(Dynamic) -&gt; Result(a, List(DecodeError))\n Occasionally you might find yourself in the situation where a JSON string is\n embedded in the dynamic value you&#39;re trying to decode. This decoder lets you\n extract that JSON and then run the decoder against it.\n\nSynonyms:\ndecipher.json_string\ndecipher json_string","url":"decipher.html#json_string"},{"doc":"decipher","title":"keys","content":"pub fn keys(\n  dynamic: Dynamic,\n) -&gt; Result(List(String), List(DecodeError))\n Decode just the keys of an object as a list of strings.\n\nSynonyms:\ndecipher.keys\ndecipher keys","url":"decipher.html#keys"},{"doc":"decipher","title":"non_empty_list","content":"pub fn non_empty_list(\n  decode: fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n) -&gt; fn(Dynamic) -&gt; Result(List(a), List(DecodeError))\n Decode a list or [arraylike](#arraylike) with at least one item into a `List`.\n If the incoming list is empty, decoding will fail.\n\nSynonyms:\ndecipher.non_empty_list\ndecipher non_empty_list","url":"decipher.html#non_empty_list"},{"doc":"decipher","title":"non_empty_string","content":"pub fn non_empty_string(\n  dynamic: Dynamic,\n) -&gt; Result(String, List(DecodeError))\n This decoder will decode a string and then confirm that it is not empty.\n\nSynonyms:\ndecipher.non_empty_string\ndecipher non_empty_string","url":"decipher.html#non_empty_string"},{"doc":"decipher","title":"non_negative_int","content":"pub fn non_negative_int(\n  dynamic: Dynamic,\n) -&gt; Result(Int, List(DecodeError))\n Decode an `Int` as long as it is greater than or equal to zero.\n\nSynonyms:\ndecipher.non_negative_int\ndecipher non_negative_int","url":"decipher.html#non_negative_int"},{"doc":"decipher","title":"number","content":"pub fn number(\n  dynamic: Dynamic,\n) -&gt; Result(Float, List(DecodeError))\n This decoder is capable of decoding both `Int` and `Float` values. If the\n value is an `Int`, it will be converted to a `Float` automatically.\n\nSynonyms:\ndecipher.number\ndecipher number","url":"decipher.html#number"},{"doc":"decipher","title":"number_string","content":"pub fn number_string(\n  dynamic: Dynamic,\n) -&gt; Result(Float, List(DecodeError))\n Decode numbers that have been converted to strings. This decoder is capable\n of decoding both `Int` and `Float` values converted to strings. Some JSON\n APIs will send numbers as strings, so this decoder can come in handy more\n often than you&#39;d think!\n\nSynonyms:\ndecipher.number_string\ndecipher number_string","url":"decipher.html#number_string"},{"doc":"decipher","title":"semver","content":"pub fn semver(\n  dynamic: Dynamic,\n) -&gt; Result(Version, List(DecodeError))\n Decode a string representing a valid semantic version into a [`Version`](https://hexdocs.pm/stoiridh_version/stoiridh/version.html#Version)\n value from the stoiridh_version package.\n\nSynonyms:\ndecipher.semver\ndecipher semver","url":"decipher.html#semver"},{"doc":"decipher","title":"set","content":"pub fn set(\n  decoder: fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n) -&gt; fn(Dynamic) -&gt; Result(Set(a), List(DecodeError))\n Decode a list or [arraylike](#arraylike) into a `Set`. Any duplicate values\n will be _dropped_. If you want to ensure that there are no duplicates, use\n the [exact_set](#exact_set) decoder instead.\n\nSynonyms:\ndecipher.set\ndecipher set","url":"decipher.html#set"},{"doc":"decipher","title":"tagged_union","content":"pub fn tagged_union(\n  tag: fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n  variants: List(\n    #(a, fn(Dynamic) -&gt; Result(b, List(DecodeError))),\n  ),\n) -&gt; fn(Dynamic) -&gt; Result(b, List(DecodeError))\n There is no standard way to represent something like Gleam&#39;s custom types as\n JSON or YAML (or most common formats). It&#39;s common then to represent them as\n a _tagged_ or _discriminated_ union where a field is used to signify which\n variant of the type is being represented.\n\n This decoder lets you decode things in this format by first decoding the tag\n and then selecting the appropriate decoder to run based on that tag.\n\n \n import decipher\n import gleam/dynamic.{type DecodeError, type Decoder, type Dynamic, DecodeError}\n\n type Example {\n   Wibble(foo: Int)\n   Wobble(bar: String)\n }\n\n fn example_decoder(dynamic: Dynamic) -&gt; Result(Example, List(DecodeError)) {\n   decipher.tagged_union(\n     dynamic.field(&quot;$&quot;, dynamic.string),\n     [\n       #(&quot;wibble&quot;, dynamic.decode1(Wibble, dynamic.field(&quot;foo&quot;, dynamic.int))),\n       #(&quot;wobble&quot;, dynamic.decode1(Wobble, dynamic.field(&quot;bar&quot;, dynamic.string))),\n     ]\n   )(dynamic)\n }\n \n\nSynonyms:\ndecipher.tagged_union\ndecipher tagged_union","url":"decipher.html#tagged_union"},{"doc":"decipher","title":"unix_timestamp","content":"pub fn unix_timestamp(\n  dynamic: Dynamic,\n) -&gt; Result(Time, List(DecodeError))\n Decode a [Unix timestamp](https://en.wikipedia.org/wiki/Unix_time) as a\n [`Time`](https://hexdocs.pm/birl/birl.html#Time) value from the birl package.\n\nSynonyms:\ndecipher.unix_timestamp\ndecipher unix_timestamp","url":"decipher.html#unix_timestamp"},{"doc":"decipher","title":"uri","content":"pub fn uri(dynamic: Dynamic) -&gt; Result(Uri, List(DecodeError))\n Decode a string representing a [URI](https://en.wikipedia.org/wiki/Uniform_Resource_Identifier)\n into a Gleam [`Uri`](https://hexdocs.pm/gleam_stdlib/gleam/uri.html#Uri) value.\n\nSynonyms:\ndecipher.uri\ndecipher uri","url":"decipher.html#uri"},{"doc":"decipher","title":"when","content":"pub fn when(\n  decoder: fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n  is predicate: fn(a) -&gt; Bool,\n) -&gt; fn(Dynamic) -&gt; Result(a, List(DecodeError))\n Run a decoder but only keep the result if it satisfies the given predicate.\n This is how decoders like [`non_negative_int`](#non_negative_int) can be\n implemented:\n\n \n import decipher\n import gleam/dynamic.{type DecodeError, type Decoder, type Dynamic, DecodeError}\n\n pub fn non_negative_int(dynamic: Dynamic) -&gt; Result(Int, List(DecodeError)) {\n   decipher.when(dynamic.int, is: fn(x) { x &gt;= 0 })(dynamic)\n }\n \n\nSynonyms:\ndecipher.when\ndecipher when","url":"decipher.html#when"},{"doc":"decipher","title":"decipher","content":"","url":"decipher.html"}]);